(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    127655,       2340]
NotebookOptionsPosition[    124181,       2226]
NotebookOutlinePosition[    125777,       2272]
CellTagsIndexPosition[    125690,       2267]
WindowTitle->DrosteEffect - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:DrosteEffect/guide/DrosteEffect"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["DROSTEEFFECT PACLET SYMBOL", "PacletNameCell"], Cell[TextData[Cell[
    BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->False], {"\<\"DrosteEffect/ref/DrosteEffect\"\>":>
      None, "\<\"Copy Mathematica url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["DrosteEffect/ref/DrosteEffect"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/DrosteEffect/ref/\
DrosteEffect.html"], StandardForm]], "Input", TextClipboardType -> 
             "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "DrosteEffect/ref/DrosteEffect", ".html"]], None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["DrosteEffect", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"DrosteEffect", "[", 
       RowBox[{
        StyleBox["img", "TI"], ",", 
        StyleBox["opt\[Ellipsis]", "TI"]}], 
       StyleBox["]", "TI"]}]], "InlineFormula"],
     " \[LineSeparator]DrosteEffect calculates a so-called Droste \
transformation of an image."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1679914183],

Cell[TextData[{
 "This is a radial version of the transform which transforms the region \
between two circles ",
 Cell[BoxData[
  SubscriptBox["r", "1"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox["r", "2"]], "InlineFormula"],
 " around a center ",
 Cell[BoxData["m"], "InlineFormula"],
 " inside the image."
}], "Notes",
 CellID->1067943069],

Cell["\<\
If not specified otherwise the output image has the ImageSize like the input \
image\
\>", "Notes",
 CellID->1531238737],

Cell["\<\
Currently three different implementations are available differing in \
execution speed: Automatic, Compiled, WolframLinkLibrary\
\>", "Notes",
 CellID->1892139274]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->693110396],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->1914267191],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Needs", "[", "\"\<DrosteEffect`\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DrosteEffect", "[", 
  GraphicsBox[
   TagBox[RasterBox[CompressedData["
1:eJxtu2dXW+m6LXhGj/7Qf6NHj76399mVXA7knHPOSAiBJARCOeecc845IwmE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     "], {{0, 128}, {128, 0}}, {0, 255},
     ColorFunction->GrayLevel],
    BoxForm`ImageTag["Byte", ColorSpace -> "Grayscale", Interleaving -> None],
    
    Selectable->False],
   BaseStyle->"ImageGraphics",
   ImageSizeRaw->{128, 128},
   PlotRange->{{0, 128}, {0, 128}}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->1043340559],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJxVu2V3m9myLXzGHe+H+zfu13dD7w6ZmZnZMkqyZTGTZTEzWmBmtkwysx07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    "], {{0, 128}, {128, 0}}, {0, 255},
    ColorFunction->GrayLevel],
   BoxForm`ImageTag["Byte", ColorSpace -> Automatic, Interleaving -> None],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSizeRaw->{128, 128},
  PlotRange->{{0, 128}, {0, 128}}]], "Output",
 ImageSize->{132, 128},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->328281288]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(4)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "DrosteCenter",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->1757724783],

Cell["\<\
Specifies the center of the transformation in pixel coordingates\
\>", "ExampleText",
 CellID->905904638]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "DrosteImplementation",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->1295379749],

Cell["Specifies the implementation to use for calculation:", "ExampleText",
 CellID->180763114],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Needs", "[", "\"\<DrosteEffect`\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"AbsoluteTiming", "@", 
     RowBox[{"DrosteEffect", "[", 
      RowBox[{
       GraphicsBox[
        TagBox[RasterBox[CompressedData["
1:eJxtu2dXW+m6LXhGj/7Qf6NHj76399mVXA7knHPOSAiBJARCOeecc845IwmE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          "], {{0, 128}, {128, 0}}, {0, 255},
          ColorFunction->GrayLevel],
         BoxForm`ImageTag[
         "Byte", ColorSpace -> "Grayscale", Interleaving -> None],
         Selectable->False],
        BaseStyle->"ImageGraphics",
        ImageSizeRaw->{128, 128},
        PlotRange->{{0, 128}, {0, 128}}], ",", 
       RowBox[{"DrosteImplementation", "\[Rule]", "#"}]}], "]"}]}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"Automatic", ",", "Compiled", ",", "WolframLinkLibrary"}], 
    "}"}]}], "//", "Grid"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->321013472],

Cell[BoxData[
 TagBox[GridBox[{
    {"1.179843`6.523369213708836", 
     GraphicsBox[
      TagBox[RasterBox[CompressedData["
1:eJxVu2V3m9myLXzGHe+H+zfu13dD7w6ZmZnZMkqyZTGTZTEzWmBmtkwysx07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        "], {{0, 128}, {128, 0}}, {0, 255},
        ColorFunction->GrayLevel],
       BoxForm`ImageTag["Byte", ColorSpace -> Automatic, Interleaving -> None],
       Selectable->False],
      BaseStyle->"ImageGraphics",
      ImageSizeRaw->{128, 128},
      PlotRange->{{0, 128}, {0, 128}}]},
    {"0.469865`6.123518089353174", 
     GraphicsBox[
      TagBox[RasterBox[CompressedData["
1:eJxVu2V3m9myLXzGHe+H+zfu13dD7w6ZmZnZMkqyZTGTZTEzWmBmtkwysx07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        "], {{0, 128}, {128, 0}}, {0, 255},
        ColorFunction->GrayLevel],
       BoxForm`ImageTag["Byte", ColorSpace -> Automatic, Interleaving -> None],
       Selectable->False],
      BaseStyle->"ImageGraphics",
      ImageSizeRaw->{128, 128},
      PlotRange->{{0, 128}, {0, 128}}]},
    {"0.020767`4.7689187563985085", 
     GraphicsBox[
      TagBox[RasterBox[CompressedData["
1:eJxVu1dbm1mzJryvueZgDubk+wlzNHPNzH7fbmdyzjkLJLKQQCjnhCLKOYEI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        "], {{0, 128}, {128, 0}}, {0, 255},
        ColorFunction->GrayLevel],
       BoxForm`ImageTag["Byte", ColorSpace -> Automatic, Interleaving -> None],
       Selectable->False],
      BaseStyle->"ImageGraphics",
      ImageSizeRaw->{128, 128},
      PlotRange->{{0, 128}, {0, 128}}]}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 ImageSize->{204, 396},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->1427739037]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "DrosteRadii",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->799913673],

Cell["Specifies the two radii to use", "ExampleText",
 CellID->21069188]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "DrosteRegion",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->309488229],

Cell["Specifies the region of the output image", "ExampleText",
 CellID->1096103930]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"DrosteEffect - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2010, 12, 6, 0, 32, 20.358968}", "context" -> "DrosteEffect`",
     "keywords" -> {}, "index" -> True, "label" -> 
    "DrosteEffect Paclet Symbol", "language" -> "en", "paclet" -> 
    "DrosteEffect", "status" -> "None", "summary" -> 
    "DrosteEffect[img, opt...] DrosteEffect calculates a so-called Droste \
transformation of an image.", "synonyms" -> {}, "title" -> "DrosteEffect", 
    "type" -> "Symbol", "uri" -> "DrosteEffect/ref/DrosteEffect"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Linux x86 (64-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Linux x86 (64-bit) (November 7, 2010)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[4706, 149, 461, 13, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->693110396]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 125547, 2260}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[605, 21, 76, 1, 3, "SymbolColorBar"],
Cell[684, 24, 190, 5, 19, "LinkTrail"],
Cell[877, 31, 1894, 42, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2796, 77, 55, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[2854, 80, 527, 16, 87, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[3418, 101, 581, 17, 40, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1679914183],
Cell[4002, 120, 358, 12, 70, "Notes",
 CellID->1067943069],
Cell[4363, 134, 130, 4, 70, "Notes",
 CellID->1531238737],
Cell[4496, 140, 173, 4, 70, "Notes",
 CellID->1892139274]
}, Closed]],
Cell[CellGroupData[{
Cell[4706, 149, 461, 13, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->693110396],
Cell[CellGroupData[{
Cell[5192, 166, 149, 5, 33, "ExampleSection",
 CellID->1914267191],
Cell[CellGroupData[{
Cell[5366, 175, 20672, 346, 62, "Input",
 CellID->1043340559],
Cell[26041, 523, 18838, 316, 149, "Output",
 CellID->328281288]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[44928, 845, 223, 7, 34, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[45176, 856, 234, 7, 70, "ExampleSubsection",
 CellID->1757724783],
Cell[45413, 865, 115, 3, 70, "ExampleText",
 CellID->905904638]
}, Closed]],
Cell[CellGroupData[{
Cell[45565, 873, 242, 7, 70, "ExampleSubsection",
 CellID->1295379749],
Cell[45810, 882, 95, 1, 70, "ExampleText",
 CellID->180763114],
Cell[CellGroupData[{
Cell[45930, 887, 21008, 355, 70, "Input",
 CellID->321013472],
Cell[66941, 1244, 56461, 944, 417, "Output",
 CellID->1427739037]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[123451, 2194, 232, 7, 70, "ExampleSubsection",
 CellID->799913673],
Cell[123686, 2203, 72, 1, 70, "ExampleText",
 CellID->21069188]
}, Closed]],
Cell[CellGroupData[{
Cell[123795, 2209, 233, 7, 70, "ExampleSubsection",
 CellID->309488229],
Cell[124031, 2218, 84, 1, 70, "ExampleText",
 CellID->1096103930]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[124154, 2224, 23, 0, 40, "FooterCell"]
}
]
*)

(* End of internal cache information *)

